<?php

namespace Drupal\new_relic_insights\Logger;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Component\Uuid\Php;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LogMessageParserInterface;
use Drupal\Core\Logger\RfcLoggerTrait;
use Drupal\Core\Queue\QueueFactory;
use Drupal\Core\Session\AccountInterface;
use Psr\Log\LoggerInterface;

/**
 * Redirects logging messages to new_relic_insights.
 */
class NewRelicInsightsLogger implements LoggerInterface {
  use RfcLoggerTrait;

  /**
   * A configuration object containing new_relic_insights settings.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * The message's placeholders parser.
   *
   * @var \Drupal\Core\Logger\LogMessageParserInterface
   */
  protected $parser;

  /**
   * The queue service.
   *
   * @var \Drupal\Core\Queue\QueueFactory
   */
  protected $queueFactory;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * Uuid service.
   *
   * @var \Drupal\Component\Uuid\Php
   */
  protected $uuid;

  /**
   * Stores whether there is a system logger connection opened or not.
   *
   * @var bool
   */
  protected $connectionOpened = FALSE;

  /**
   * Constructs a new_relic_insights object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory object.
   * @param \Drupal\Core\Logger\LogMessageParserInterface $parser
   *   The parser to use when extracting message variables.
   * @param \Drupal\Core\Queue\QueueFactory $queue_factory
   *   The queue service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Component\Uuid\Php $uuid
   *   The UUID service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, LogMessageParserInterface $parser, QueueFactory $queue_factory, AccountInterface $current_user, Php $uuid) {
    $this->config = $config_factory->get('new_relic_insights.settings');
    $this->parser = $parser;
    $this->queueFactory = $queue_factory;
    $this->currentUser = $current_user;
    $this->uuid = $uuid;
  }

  /**
   * {@inheritdoc}
   */
  public function log($level, $message, array $context = []) {
    // Bail out if watchdog integration is disabled.
    if (!$this->config->get('collect_watchdog')) {
      return;
    }

    $type = 'watchdog';

    // Convert PSR3-style messages to \Drupal\Component\Render\FormattableMarkup
    // style, so they can be used to render the entire message with placeholder
    // replacements.
    $message_placeholders = $this->parser->parseMessagePlaceholders($message, $context);

    // Add an iid parameter so Insight entities can be loaded individually.
    $data['iid'] = $this->uuid->generate();
    $data['type'] = mb_substr($context['channel'], 0, 64);
    // Decorate the log entry with the current user.
    $data['user'] = $this->currentUser->getEmail() ?? 'anonymous';
    $data['message'] = $message;
    // Apply the placeholder values to the message.
    $data['fullMessage'] = (string) new FormattableMarkup($message, $message_placeholders);
    $data['severity'] = $level;
    $data['link'] = $context['link'];
    $data['request_uri'] = $context['request_uri'];
    $data['referer'] = $context['referer'];
    $data['hostname'] = mb_substr($context['ip'], 0, 128);
    $data['timestamp'] = $context['timestamp'];

    // Store each variable separately, and prefix with `_var_` .
    foreach ($message_placeholders as $variable => $value) {
      $sanitized_variable = str_replace([':', '%', '@'], '_var_', $variable);
      $data[$sanitized_variable] = (string) $value;
    }

    $this->logWrapper($type, $data);
  }

  /**
   * Wrapper to make new_relic_insights functionality testable.
   *
   * @param string $type
   *   Log message type.
   * @param array $data
   *   Message array.
   */
  protected function logWrapper($type, array $data) {
    // Notify the New Relic Agent of the event if possible.
    if (function_exists('newrelic_record_custom_event')) {
      newrelic_record_custom_event($type, $data);
    }
    // Otherwise queue the event for posting via the Insert API.
    else {
      // Set the "eventType" key. This is required for the Insert API.
      $data['eventType'] = $type;
      /** @var \Drupal\Core\Queue\QueueInterface $queue */
      $queue = $this->queueFactory->get('new_relic_insights');
      $queue->createItem($data);
    }
  }

}
