<?php

namespace Drupal\new_relic_insights\Plugin\QueueWorker;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Processes items for posting to New Relic.
 *
 * @QueueWorker(
 *   id = "new_relic_insights",
 *   title = @Translation("Post item: New Relic Insights queue"),
 *   cron = {"time" = 60}
 * )
 */
class NewRelicInsightsQueue extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * A configuration object containing new_relic_insights settings.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  public function __construct($plugin_id, $plugin_definition, ConfigFactoryInterface $config_factory) {
    $this->config = $config_factory->get('new_relic_insights.settings');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    $this->newRelicWrapper($data);
  }

  /**
   * Wrapper to make new_relic_insights functionality testable.
   *
   * @param string $data
   *   The message to send to new_relic_insights function.
   */
  protected function newRelicWrapper($data) {
    // Only proceed if an account ID and insert key are present.
    $account_id = $this->config->get('account_id');
    $insert_key = $this->config->get('insert_key');
    if ($account_id && $insert_key) {
      // Build the Insights Insert API URL.
      $url = $this->config->get('base_url');
      $url .= '/accounts/' . $account_id;
      $url .= '/events';

      // JSON encode the data.
      $data = json_encode($data);

      // Initialize, configure, and execute the curl handle.
      $ch = curl_init($url);
      curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
      curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
      curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $this->config->get('curl_timeout'));
      curl_setopt($ch, CURLOPT_HTTPHEADER, [
        'Content-Type: application/json',
        'Content-Length: ' . strlen($data),
        'User-Agent: ' . $this->config->get('user_agent'),
        'X-Insert-Key: ' . $insert_key,
      ]);
      curl_exec($ch);
      curl_close($ch);
    }
  }

}
