<?php

namespace Drupal\new_relic_insights\Plugin\Statistics\ActionHandler;

use Drupal\Component\Uuid\Php;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Queue\QueueFactory;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\sapi\ActionHandlerBase;
use Drupal\sapi\ActionTypeInterface;
use Drupal\sapi\Plugin\Statistics\ActionType\EntityInteraction;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\Role;

/**
 * Action handler to track any entity interactions (view, update and create).
 *
 * @ActionHandler(
 *  id = "new_relic_insights_entity_interaction_handler",
 *  label = "New Relic Insights Entity Interaction Handler",
 *  description = "Log entity interactions to New Relic Insights"
 * )
 */
class NewRelicInsightsEntityInteraction extends ActionHandlerBase implements ContainerFactoryPluginInterface {

  use StringTranslationTrait;

  /**
   * EntityTypeManager used to create and edit sapi_data items from tracking.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The queue service.
   *
   * @var \Drupal\Core\Queue\QueueFactory
   */
  protected $queueFactory;

  /**
   * Uuid service.
   *
   * @var \Drupal\Component\Uuid\Php
   */
  protected $uuid;

  /**
   * EntityInteractionTracker constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Config factory required by ConfigurableActionHandlerBase.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   EntityTypeManager to load sapi data items.
   * @param \Drupal\Core\Queue\QueueFactory $queue_factory
   *   The queue service.
   * @param \Drupal\Component\Uuid\Php $uuid
   *   The UUID service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $configFactory, EntityTypeManagerInterface $entityTypeManager, QueueFactory $queue_factory, Php $uuid) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $configFactory);

    $this->entityTypeManager = $entityTypeManager;
    $this->queueFactory = $queue_factory;
    $this->uuid = $uuid;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('queue'),
      $container->get('uuid')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'roles' => ['authenticated'],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    /** @var array $all_roles */
    $all_roles = [];
    foreach (Role::loadMultiple() as $role) {
      /** @var \Drupal\user\Entity\Role $role */
      $all_roles[$role->id()] = $role->label();
    }
    $form['roles'] = [
      '#type' => 'checkboxes',
      '#title' => 'Tracked roles',
      '#description' => $this->t('Track requests only for the selected role(s). If you select no roles, statistics will be collected for all users.'),
      '#options' => $all_roles,
      '#default_value' => $this->configuration['roles'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function process(ActionTypeInterface $action) {
    // Only acts if $action is an EntityInteraction plugin type.
    if (!($action instanceof EntityInteraction) ||
      (!empty($this->configuration['roles']) &&
      empty(array_intersect($action->getAccount()->getRoles(), $this->configuration['roles'])))) {
      return;
    }

    /** @var \Drupal\Component\Plugin\Context\ContextInterface[] $contexts */
    $contexts = $action->getContextValues();

    /** @var string $entity_type */
    $entity_type = $contexts['entity']->getEntityTypeId();

    /** @var string $entity_id */
    $entity_id = $contexts['entity']->id();

    /** @var string $interaction_type */
    $interaction_type = $contexts['action'];

    /** @var string $user_id */
    $user_id = $contexts['account']->id();


    $type = 'entity_interaction';
    // Add an iid parameter so Insight entities can be loaded individually.
    $data['iid'] = $this->uuid->generate();
    $data['entity_type'] = $entity_type;
    $data['entity_id'] = $entity_id;
    $data['interaction_type'] = $interaction_type;
    $data['user_id'] = $user_id;

    $this->processWrapper($type, $data);
  }

  /**
   * Wrapper to make new_relic_insights functionality testable.
   *
   * @param string $type
   *   Log message type.
   * @param array $data
   *   Message array.
   */
  protected function processWrapper($type, array $data) {
    // Notify the New Relic Agent of the event if possible.
    if (function_exists('newrelic_record_custom_event')) {
      newrelic_record_custom_event($type, $data);
    }
    // Otherwise queue the event for posting via the Insert API.
    else {
      // Set the "eventType" key. This is required for the Insert API.
      $data['eventType'] = $type;
      /** @var \Drupal\Core\Queue\QueueInterface $queue */
      $queue = $this->queueFactory->get('new_relic_insights');
      $queue->createItem($data);
    }
  }

}
